#============================================================================================================
#
#	ANZX[UǗW[
#
#============================================================================================================
package	FARAMIR;

use strict;
#use warnings;

#------------------------------------------------------------------------------------------------------------
#
#	W[RXgN^ - new
#	-------------------------------------------
#	@FȂ
#	߂lFW[IuWFNg
#
#------------------------------------------------------------------------------------------------------------
sub new
{
	my $class = shift;
	
	my $obj = {
		'TYPE'		=> undef,
		'METHOD'	=> undef,
		'USER'		=> undef,
		'SYS'		=> undef,
	};
	bless $obj, $class;
	
	return $obj;
}

#------------------------------------------------------------------------------------------------------------
#
#	[Uf[^ǂݍ - Load
#	-------------------------------------------
#	@F$Sys : MELKOR
#	߂lFǂݍ:0,G[:1
#
#------------------------------------------------------------------------------------------------------------
sub Load
{
	my $this = shift;
	my ($Sys) = @_;
	
	$this->{'SYS'} = $Sys;
	$this->{'USER'} = [];
	
	my $path = $Sys->Get('BBSPATH') . '/' . $Sys->Get('BBS') . "/info/access.cgi";
	
	if (open(my $fh, '<', $path)) {
		flock($fh, 2);
		my @datas = <$fh>;
		close($fh);
		map { s/[\r\n]+\z// } @datas;
		
		my @head = split(/<>/, shift(@datas), -1);
		$this->{'TYPE'} = $head[0];
		$this->{'METHOD'} = $head[1];
		
		push @{$this->{'USER'}}, @datas;
		return 0;
	}
	return 1;
}

#------------------------------------------------------------------------------------------------------------
#
#	[Uf[^ - Save
#	-------------------------------------------
#	@F$Sys : MELKOR
#	߂lF폑:0,G[:-1
#
#------------------------------------------------------------------------------------------------------------
sub Save
{
	my $this = shift;
	my ($Sys) = @_;
	
	my $path = $Sys->Get('BBSPATH') . '/' . $Sys->Get('BBS') . "/info/access.cgi";
	
	chmod($Sys->Get('PM-ADM'), $path);
	if (open(my $fh, (-f $path ? '+<' : '>'), $path)) {
		flock($fh, 2);
		seek($fh, 0, 0);
		binmode($fh);
		
		print $fh "$this->{'TYPE'}<>$this->{'METHOD'}\n";
		foreach (@{$this->{'USER'}}) {
			print $fh "$_\n";
		}
		
		truncate($fh, tell($fh));
		close($fh);
	}
	chmod($Sys->Get('PM-ADM'), $path);
	
	return 0;
}

#------------------------------------------------------------------------------------------------------------
#
#	[Uǉ - Set
#	-------------------------------------------
#	@F$name : ǉ[U
#	߂lFȂ
#
#------------------------------------------------------------------------------------------------------------
sub Add
{
	my $this = shift;
	my ($name) = @_;
	
	push @{$this->{'USER'}}, $name;
}

#------------------------------------------------------------------------------------------------------------
#
#	[Uf[^擾 - Get
#	-------------------------------------------
#	@F$key : 擾L[
#			$default : ftHg
#	߂lF[Uf[^
#
#------------------------------------------------------------------------------------------------------------
sub Get
{
	my $this = shift;
	my ($key, $default) = @_;
	
	my $val = $this->{$key};
	
	return (defined $val ? $val : (defined $default ? $default : undef));
}

#------------------------------------------------------------------------------------------------------------
#
#	[UNA - Clear
#	-------------------------------------------
#	@FȂ
#	߂lFȂ
#
#------------------------------------------------------------------------------------------------------------
sub Clear
{
	my $this = shift;
	
	$this->{'USER'} = [];
}

#------------------------------------------------------------------------------------------------------------
#
#	[Uf[^ݒ - SetData
#	-------------------------------------------
#	@F$key  : ݒL[
#			$data : ݒf[^
#	߂lFȂ
#
#------------------------------------------------------------------------------------------------------------
sub Set
{
	my $this = shift;
	my ($key, $data) = @_;
	
	$this->{$key} = $data;
}

#------------------------------------------------------------------------------------------------------------
#
#	[U - Check
#	-------------------------------------------
#	@F$host : zXg
#			$addr : IPAhX
#			$koyuu : [ŗLʎq
#	߂lFo^[U:1,o^[U:0
#
#------------------------------------------------------------------------------------------------------------
sub Check
{
	my $this = shift;
	my ($host, $addr, $koyuu) = @_;
	
	my $Sys = $this->{'SYS'};
	my $addrb = unpack('B32', pack('C*', split(/\./, $addr)));
	my $flag = 0;
	my $adex = '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}';
	
	foreach my $line (@{$this->{'USER'}}) {
		next if ($line =~ /^[#;]|^$/);
		
		# IPAhX/CIDR
		if ($line =~ m|^($adex)(?:/([0-9]+))?$|) {
			my $leng = $2 || 32;
			my $a = unpack("B$leng", pack('C*', split(/\./, $1)));
			if (substr($addrb, 0, $leng) eq $a) {
				$flag = 1;
				$Sys->Set('HITS', $line);
				last;
			}
		}
		# IPAhX͈͎w
		elsif ($line =~ m|^($adex)-($adex)$|) {
			my $a = unpack('B32', pack('C*', split(/\./, $1)));
			my $b = unpack('B32', pack('C*', split(/\./, $2)));
			($b, $a) = ($a, $b) if ($a gt $b);
			if ($addrb ge $a && $addrb le $b) {
				$flag = 1;
				$Sys->Set('HITS', $line);
				last;
			}
		}
		# [ŗLʎq
		elsif (defined $koyuu && $koyuu =~ /^\Q$line\E$/) {
			$flag = 1;
			$Sys->Set('HITS', $line);
			last;
		}
		# zXg(K\)
		elsif ($host =~ /$line/) {
			$flag = 1;
			$Sys->Set('HITS', $line);
			last;
		}
	}
	
	# K[U
	if ($flag && $this->{'TYPE'} eq 'disable') {
		if ($this->{'METHOD'} eq 'disable') {
			# Fݕs
			return 4;
		}
		elsif ($this->{'METHOD'} eq 'host') {
			# FzXg\
			return 2;
		}
		else {
			return 4;
		}
	}
	# 胆[UȊO
	elsif (! $flag && $this->{'TYPE'} eq 'enable') {
		if ($this->{'METHOD'} eq 'disable') {
			# Fݕs
			return 4;
		}
		elsif ($this->{'METHOD'} eq 'host') {
			# FzXg\
			return 2;
		}
		else {
			return 4;
		}
	}
	return 0;
}

#============================================================================================================
#	W[I[
#============================================================================================================
1;
